# The MIT License
# 
# Copyright (c) Paul Crawford
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

require 'redis'
require 'json'

# CoffeeQClient
# ============
#
# Usage:
# require 'coffeeq_client'
# r = CoffeeQClient.new
# r.enqueue('queue1', 'add', [4,5])
# r.dequeue("queue1") { |msg| puts "got message #{msg}"}
#
# Client publishing:
# 1. push an item onto a queue 
# 2. send a pub sub message on the queue channel
# 
# Client working: 
# 1. subscribe to queue channel
# 2. on receipt of message pop item from queue
# 
class CoffeeQClient
  attr_accessor :client
  
  def initialize(host='localhost', port=6379)
    @host = host
    @port = port
    
    @pubsubClient = Redis.new(:host => @host, :port => @port)    
    @queueClient = Redis.new(:host => @host, :port => @port)
  end
  
  def enqueue(queue, func, args)
    puts "enqueue"
    val = {:class => func, :args => args}.to_json
    key = ":queue:#{queue}"
    puts "key #{key}"
    @queueClient.rpush key, val
    @pubsubClient.publish key, "queued" do
      puts "published"
    end
  end
  
  def dequeue(queue, &block)
    queue_channel = ":queue:#{queue}"
    trap(:INT) { puts; exit }
    
    @pubsubClient.subscribe(queue_channel) do |on|
      on.subscribe do |channel, subscriptions|
        puts "Subscribed to ##{channel} (#{subscriptions} subscriptions)"
      end

      on.message do |channel, message|
        puts "##{channel}: #{message}"
        puts @queueClient.llen(channel)
        puts @queueClient.lpop(channel)
        block.call(message)
      end

      on.unsubscribe do |channel, subscriptions|
        puts "Unsubscribed from ##{channel} (#{subscriptions} subscriptions)"
      end
    end
  end  
end