(function() {
  var CoffeeQ, Worker, redis;
  var __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; }, __slice = Array.prototype.slice;
  redis = require('redis');
  Worker = require('./coffeeq_worker');
  /*
    CoffeeQ
    Uses two redis client instances so that it can handle pubsub and normal events simultaneously (redis connections have a pubsub mode that prevents one from doing both pubsub and normal events simultaneously)
    @queueClient, @pubsubClient
  */
  CoffeeQ = (function() {
    function CoffeeQ(options) {
      console.log("Init CoffeeQ");
      if (!options) {
        options = {};
      }
      this.port = options.port || 6379;
      this.host = options.host || 'localhost';
      this.queueClient = redis.createClient(this.port, this.host);
      this.pubsubClient = redis.createClient(this.port, this.host);
    }
    /*
      Public: Queues a job in a given queue to be run.
      
       queue - String queue name.
       func  - String name of the function to run.
       args  - Optional Array of arguments to pass.
      
       Returns nothing.      
      */
    CoffeeQ.prototype.enqueue = function(queue, func, args) {
      var key, val;
      console.log("enqueue");
      val = JSON.stringify({
        "class": func,
        args: args
      });
      key = this.key('queue', queue);
      console.log("key " + key);
      return this.queueClient.rpush(key, val, __bind(function(err, val) {
        console.log("pushed " + val);
        if (!err) {
          return this.pubsubClient.publish(key, "queued", function() {
            return console.log("published on " + queue);
          });
        }
      }, this));
    };
    /*
        Builds a namespaced Redis key with the given arguments.
        args - Array of Strings.  
        Returns an assembled String key
      */
    CoffeeQ.prototype.key = function() {
      var args;
      args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      args.unshift(this.namespace);
      return args.join(":");
    };
    return CoffeeQ;
  })();
  CoffeeQ.Worker = Worker;
  CoffeeQ.version = "0.0.1";
  module.exports = CoffeeQ;
}).call(this);
