root = exports ? window

root.StateMachine = class StateMachine

  # @stateMachine =
  #   states:
  #     stateName: 
  #       active:  isActive
  #       onEnter: enterMethod
  #       onExit:  exitMethod
  #       guard:   guardMethod
  #   events:
  #     eventName:
  #       from: fromState
  #       to:   toState

  constructor: (@stateMachine = {}) ->
    # If array setup was used, translate it into the object setup
    if @stateMachine.states.constructor.toString().indexOf('Array') isnt -1
      states = @stateMachine.states
      @stateMachine.states = {}
      for state in states
        @stateMachine.states[state] = { active: (state == states[0]) }
    
    # Make sure an active state is properly set
    activeStates = (state for own state, stateDef of @stateMachine.states when stateDef.active)
    if activeStates.length is 0
      # Set the 1st state to active
      for own state, stateDef of @stateMachine.states
        stateDef.active = true
        break
    else if activeStates.length > 1
      # Set only the 1st active state to active
      for own state in activeStates
        continue if state is activeStates[0]
        stateDef.active = false
    
    # Define the event methods
    for event, eventDef of @stateMachine.events
      do (event, eventDef) =>        
        this[event] = -> this.changeState(eventDef.from, eventDef.to)
    
  currentState: ->
    (state for own state, stateDef of @stateMachine.states when stateDef.active)[0]

  availableStates: ->
    state for own state of @stateMachine.states
    
  availableEvents: ->
    event for own event of @stateMachine.events
    
  changeState: (from, to) ->
    console.log "From: #{from}"
    console.log "To: #{to}"
    fromStateDef = @stateMachine.states[from]
    toStateDef = @stateMachine.states[to]
    throw "Cannot change from state '#{from}'; it is undefined!" if fromStateDef is undefined
    throw "Cannot change to state '#{to}'; it is undefined!" if toStateDef is undefined
    throw "Cannot change from state '#{from}'; it is not the active state!" if fromStateDef.active isnt true

    {onExit: exitMethod} = fromStateDef
    {onEnter: enterMethod, guard: guardMethod} = toStateDef
    
    if guardMethod isnt undefined
      return false if guardMethod.call() is false
    exitMethod.call() if exitMethod isnt undefined
    enterMethod.call() if enterMethod isnt undefined
    fromStateDef.active = false
    toStateDef.active = true

#############
sm = new StateMachine
  states: ['state1', 'state2']
  events:
    state1to2:
      from: 'state1'
      to: 'state2'
    state2to1:
      from: 'state2'
      to: 'state1'

try
  console.log sm.state1to2.toString()
  sm.state1to2()
catch error
  console.log "Caught: #{error}"